/*
This ActionManager class manages all the actions that the movable objects in a game object 
cause when they interact with other objects.

The main information pass is with the game and level classes for this class.

Copyright(c) Group Seven.
Added to the OO-Ball after the 1.1 version.
*/

import java.awt.*;
import java.util.*;
import symantec.itools.multimedia.*;

class ActionManager
{
    private Game game;
    private boolean playMusic;
    private SoundPlayer soundPlayer1 = new SoundPlayer();
    private SoundPlayer soundPlayer2 = new SoundPlayer();

    ActionManager(Game parGame){
        game=parGame;
	  if(playMusic=((GreetingFrame)game.getOwner()).isPlayingMusic()){
	  try {
			soundPlayer1.setRepeat(1);
			soundPlayer2.setRepeat(1);
	  }
	  catch(java.beans.PropertyVetoException e) { }
	  try {
				java.net.URL[] tempURL = new java.net.URL[1];
				tempURL[0] = symantec.itools.net.RelativeURL.getURL("sounds/bullet.au");
				soundPlayer1.setURLList(tempURL);
				tempURL[0] = symantec.itools.net.RelativeURL.getURL("sounds/futility.au");
				soundPlayer2.setURLList(tempURL);
		}
	catch (java.net.MalformedURLException error) { }
	catch(java.beans.PropertyVetoException e) { }
	}

    }

    public void fireLaser(NormalLevel parLevel){
	if(playMusic)
		soundPlayer1.play();
	  Fire newLaser=new Fire(parLevel.paddle.getX()+Paddle.EDGE_DISTANCE,parLevel.paddle.getY()-20,Fire.LASER, new Direction(0,-1),parLevel.ownerComponent, parLevel);
        parLevel.fires.addElement(newLaser);
        parLevel.elements.addElement(newLaser);
        newLaser=new Fire(parLevel.paddle.getX()+parLevel.paddle.getWidth()-Paddle.EDGE_DISTANCE,parLevel.paddle.getY()-20,Fire.LASER, new Direction(0,-1),parLevel.ownerComponent, parLevel);
        parLevel.fires.addElement(newLaser);
        parLevel.elements.addElement(newLaser);
    }

    public void releaseBall(LevelTemplate parLevel){
        Ball nextBall;
        for (Enumeration e = parLevel.balls.elements() ; e.hasMoreElements() ;) {
                nextBall=((Ball)e.nextElement());
                if(nextBall.beingHold)
                    nextBall.beingHold=false;
        }
        if(!parLevel.ownerGame.hasLevelStarted()){
            parLevel.paddle.setType(Paddle.NORMAL);
            parLevel.start();
            parLevel.ownerGame.levelStarted(true);
            parLevel.ownerGame.notifyObserver();
        }
    }    
    public void movePaddle(LevelTemplate parLevel, int parRight, int parDown){
	  Paddle parPad=parLevel.paddle;
        movePaddle(parLevel,new Point(parPad.getX()+parPad.getWidth()/2+parRight,parPad.getY()+parDown));
    }

    public void movePaddle(LevelTemplate parLevel, Point parPosition){
        Paddle parPad=parLevel.paddle;
        int difX, difY=-parPad.getY()+parPosition.y;
        if(parPad.isFrozen())
            return;
        parPosition.x-=parPad.getWidth()/2;
        difX=-parPad.getX();
        Wall lWall=game.getCurrentLevel().walls[0];
        Wall rWall=game.getCurrentLevel().walls[2];
        int rLimit=rWall.getX();
        int lLimit=lWall.getX()+lWall.getWidth();
	  if(parPad.type==Paddle.SPLIT){
	  	rLimit-=60;
		lLimit+=60;
	  }
        if(parPosition.x<lLimit)
            parPosition.x=lLimit;
        if(parPosition.x+parPad.getWidth()>rLimit)
            parPosition.x=rLimit-parPad.getWidth();
        if(parPad.isVerticalEnabled()){
            if(parPosition.y>parLevel.position.y+parLevel.dim.height-100&&parPosition.y<parLevel.position.y+parLevel.dim.height-Paddle.HEIGHT/2){
                parPad.setY(parPosition.y);                
            }
            else
                difY=0;
        }  
        else{
            difY=0;
        }
        parPad.setX(parPosition.x);
        difX+=parPosition.x;
        if(parPad.type==parPad.HOLD)
            for (Enumeration e = parLevel.balls.elements() ; e.hasMoreElements() ;) {
                Ball nextBall=(Ball)e.nextElement();
                int size=nextBall.SIZE_OF_NORMAL_BALL;
                if(nextBall.getType()==nextBall.BIG_SIZE)
                    size+=nextBall.SIZE_DIFFERENCE;
                if(nextBall.getType()==nextBall.TINY_SIZE)
                    size-=nextBall.SIZE_DIFFERENCE;
                if(nextBall.beingHold)
                    if(parLevel.ownerGame.hasLevelStarted())
                        nextBall.setPosition(nextBall.getX()+difX,nextBall.getY()+difY);
                    else
                        nextBall.setPosition(parPad.getX()+parPad.getWidth()/2-nextBall.getWidth()/2,parPad.getY()-nextBall.getHeight());
                
            }
        
    }

    public void ballMotion(MonsterLevel parLevel, Ball parBall){
        basicBallMotion(parLevel,parBall);
        Direction locDir=parBall.getDirection();
        if(parBall.intersects(parLevel.boss)){
		if(parLevel.boss.hit()){
			parLevel.ownerGame.finishLevel();
		}
            Ball newBall=new Ball((Ball)parBall);
            newBall.setPosition(newBall.getX()-(int)locDir.x, newBall.getY());
            if(newBall.intersects(parLevel.boss)){
                parBall.position.y-=locDir.y;
                parBall.changeDirection(new Direction(locDir.x,-locDir.y));
            }
            else{
                parBall.position.x-=locDir.x;
                parBall.changeDirection(new Direction(-locDir.x,locDir.y));
            }
        }
    }
    
    public void ballMotion(NormalLevel parLevel, Ball parBall){
        int ballX=(parBall.getActualX())/Brick.WIDTH;
        int ballY=(parBall.getActualY())/Brick.HEIGHT;
        checkBricksAndBall(0,parBall,ballX,ballY,parLevel);
        MovingBrick nextMvB;
        for (Enumeration e = (parLevel).mvBricks.elements() ; e.hasMoreElements() ;) {
                nextMvB=(MovingBrick)e.nextElement();
                if(parBall.intersects(nextMvB))
                    brickBallInteraction(nextMvB, parBall);
        }
        if(!parLevel.areMonstersMoving()){
            Monster nextM;
            for (Enumeration e = (parLevel).monsters.elements() ; e.hasMoreElements() ;) {
                nextM=(Monster)e.nextElement();
                if(parBall.intersects(nextM)){
                    monsterBallInteraction(parBall,nextM,parLevel);
                    nextM.terminate();                                      
                }
            }
        }
        basicBallMotion(parLevel,parBall);        
    }
    
    public void basicBallMotion(LevelTemplate parLevel, Ball parBall){
        Direction locDir=parBall.getDirection();
        if(parBall.getActualX()<=0){
            locDir.x=Math.abs(locDir.x);
            parBall.position.x+=locDir.x;
        }
	  if(parBall.getActualX()+parBall.getWidth()>=parLevel.walls[2].getActualX()){
            locDir.x=-Math.abs(locDir.x);
            parBall.position.x+=locDir.x;
	  }
	  if(parBall.getActualY()<=0){
            locDir.y=Math.abs(locDir.y);
            parBall.position.y+=locDir.y;
        }
	  if(parBall.intersects(parLevel.paddle)){
            
		double change;
		if(parLevel.paddle.type==Paddle.SPLIT)
			if(parBall.getX()>parLevel.paddle.getX()+parLevel.paddle.getWidth()/2)
		            change=(parBall.getX()+parBall.getWidth()/2-(parLevel.paddle.getX()+60+parLevel.paddle.getWidth()/2))/5;
			else
		            change=(parBall.getX()+parBall.getWidth()/2-(parLevel.paddle.getX()-60+parLevel.paddle.getWidth()/2))/5;
		else
	            change=(parBall.getX()+parBall.getWidth()/2-(parLevel.paddle.getX()+parLevel.paddle.getWidth()/2))/5;
            if(parLevel.paddle.type!=Paddle.HOLD){
                locDir.x+=change;
                parBall.changeDirection(new Direction(locDir.x,-locDir.y));
                parBall.position.x+=locDir.x;
                parBall.position.y+=locDir.y;
            }
            else{
                parBall.beingHold=true;
                parBall.setY(parLevel.paddle.getY()-parBall.getHeight());
                parBall.changeDirection(new Direction(0+change,-1));                          
            }
            
            if(parLevel instanceof NormalLevel)
                if(((NormalLevel)parLevel).areBricksComingDown())
                    ((NormalLevel)parLevel).comeDownBricks();
        }else if(parBall.getActualY()>(parLevel.dim.height)){
            parLevel.lostBall(parBall);
	  }
        
    }

    public void checkBricksAndBall(int times,Ball parBall, int parBallX,int parBallY,LevelTemplate parLevel){
        if (times==4) 
            return;
        String xCode;
        String yCode;
        int ballX=parBallX, ballY=parBallY;
        Direction locDir=parBall.getDirection();
        Brick nextBrick;
        switch(times){
            case 1:
                ballX=parBallX+1;
                break;
            case 2:
                ballY=parBallY+1;
                break;
            case 3:
                ballX=parBallX+1;
                ballY=parBallY+1;
                break;
            default:
        }
        if(ballX<10)
            xCode="0"+ballX;
        else
            xCode=""+ballX;
        if(ballY<10)
            yCode="0"+ballY;
        else
            yCode=""+ballY;

        nextBrick=(Brick)((NormalLevel)parLevel).bricks.get(xCode+yCode);
        if(nextBrick!=null){
            if(nextBrick.intersects(parBall)){
                brickBallInteraction(nextBrick, parBall);            
            }
        }
        times++;
        checkBricksAndBall(times,parBall,parBallX,parBallY,parLevel);
    }
    
    private void brickBallInteraction(Brick nextBrick, Ball parBall){        
      Direction locDir=parBall.getDirection();
     	if(parBall.getType()!=Ball.THROUGH_TYPE){
            Ball newBall=new Ball((Ball)parBall);
            newBall.setPosition(newBall.getX()-(int)locDir.x, newBall.getY());
            if(newBall.intersects(nextBrick)){
                 	parBall.changeDirection(new Direction(locDir.x,-locDir.y));
                  parBall.position.y+=locDir.y;
            }
            else{
                  parBall.changeDirection(new Direction(-locDir.x,locDir.y));
                  parBall.position.x+=locDir.x;
            }
      }
        
      if(parBall.getType()==Ball.FIRE_TYPE){
            nextBrick.setType(Brick.EXPLOSIVE);
		nextBrick.terminate();
	}else if(parBall.getType()==Ball.THROUGH_TYPE){
     		nextBrick.hit(-1);
		nextBrick.terminate();
    	}else if(nextBrick.hit(1)){
            nextBrick.terminate();
      }                
    }
    
    public void bonusMotion(NormalLevel parLevel,Bonus parBonus){

     	if (parBonus.intersects(parLevel.paddle)){
		if(playMusic)
			soundPlayer2.play();
            parLevel.elements.removeElement(parBonus);
            parLevel.bonuses.removeElement(parBonus);
            parLevel.evalBonus(parBonus);
      }else if(parBonus.getY()>parLevel.paddle.getY()){
            parLevel.elements.removeElement(parBonus);
            parLevel.bonuses.removeElement(parBonus);
      }
            
    }

    public void brickMotion(NormalLevel parLevel, MovingBrick parMvB){
        int compX,compY;
        String xCode,yCode;
        compX=parMvB.getXCoor();
        compY=parMvB.getYCoor();
        if(parMvB.isMovingToRight())
        compX++;
        if(compX<10)
            xCode="0"+compX;
        else
            xCode=""+compX;                            
        if(compY<10)
            yCode="0"+compY;
        else
            yCode=""+compY;                            
        Brick intersectingBrick=(Brick)parLevel.bricks.get(xCode+yCode);
        MovingBrick nextMvB;
        if(intersectingBrick!=null)
            parMvB.changeDirectionToRight(!parMvB.isMovingToRight());
        else{
            boolean changed=false;
            Enumeration e;
            for ( e= (parLevel).mvBricks.elements() ; e.hasMoreElements() && !changed ;) {
                    nextMvB=(MovingBrick)e.nextElement();
                    if(nextMvB!=parMvB)
                        if(parMvB.intersects(nextMvB)){
			            parMvB.changeDirectionToRight(!parMvB.isMovingToRight());
                            changed=true;
                        }
            }
            for (e = (parLevel).monsters.elements() ; e.hasMoreElements() && !changed;) {
                    if(parMvB.intersects((Monster)e.nextElement())){
                        parMvB.changeDirectionToRight(!parMvB.isMovingToRight());
                        changed=true;
                    }
            }
        }
        if(parMvB.intersects(parLevel.walls[0]))
            parMvB.changeDirectionToRight(true);
        else if(parMvB.intersects(parLevel.walls[2]))
            parMvB.changeDirectionToRight(false);
        
        if(parMvB.isMovingToRight())
            parMvB.position.x+=MovingBrick.HORIZONTAL_DISPLACEMENT;
        else
            parMvB.position.x-=MovingBrick.HORIZONTAL_DISPLACEMENT;

    }

    public void fireMotion(MonsterLevel parLevel, Fire parF){
        if(parLevel.paddle.intersects(parF)){        
            parLevel.ownerGame.death();
        }
        else if(parF.getActualY()>parLevel.dim.height){
            parLevel.fires.removeElement(parF);
            parLevel.elements.removeElement(parF);
        }else if(parF.intersects(parLevel.walls[0])||parF.intersects(parLevel.walls[2])){
	  	parLevel.fires.removeElement(parF);
            parLevel.elements.removeElement(parF);
	  }
    }

    public void fireMotion(NormalLevel parLevel, Fire parF){
        int coorX=parF.getActualX()/Brick.WIDTH;
        int coorY=(parF.getActualY()/Brick.HEIGHT);
        String xCode, yCode;
        if(coorX<10)
            xCode="0"+coorX;
        else
            xCode=""+coorX;
        if(coorY<10)
            yCode="0"+coorY;
        else
            yCode=""+coorY;
        Brick hitBrick=(Brick)parLevel.bricks.get(xCode+yCode);
        boolean hit=false;
        if(hitBrick!=null){
            if(hitBrick.hit(1))
                hitBrick.terminate();            
                hit=true;
        }else if(parF.getY()<=parLevel.walls[1].getY()+Wall.SIZE){
                hit=true;
        }else{
            MovingBrick nextMvB;
            for (Enumeration e = (parLevel).mvBricks.elements() ; e.hasMoreElements() && !hit;) {
                    nextMvB=(MovingBrick)e.nextElement();
                    if(parF.intersects(nextMvB)){
                            if(nextMvB.hit(1))
                                nextMvB.terminate();
                            hit=true;
                    }
            }           
        }
        if(hit){
            parLevel.fires.removeElement(parF);
            parLevel.elements.removeElement(parF);
        }
    }
    
    public void monsterMotion(NormalLevel parLevel,Monster parM){
        ObjectTemplate nextObject;
        for (Enumeration e = (parLevel).elements.elements() ; e.hasMoreElements() ;) {
                nextObject=(ObjectTemplate)e.nextElement();
                if(parM!=nextObject){
                    if(parM.intersects(nextObject)){
                        if(nextObject instanceof Fire)
					parM.terminate();
				else if((nextObject instanceof Ball)||(nextObject instanceof Paddle)){
                            if(nextObject instanceof Ball){
                                monsterBallInteraction((Ball)nextObject,parM,parLevel);
                                if(!parLevel.areMonstersMoving()||(parM.getType()!=Monster.BALL_EATER&&parM.getType()!=Monster.BALL_SENDER)||((Ball)nextObject).getType()==Ball.THROUGH_TYPE)
                                    parM.terminate();                                                                  
                            }
                            else if(parM.getType()!=Monster.BALL_SENDER)
                                parM.terminate();                                                                  
                            else
  					    parM.nextDirection();
				    
                        }
                        else{
                            parM.nextDirection();
                        }
                    }
                }
        }   
        if(parM.getActualY()>parLevel.dim.height){
            ((NormalLevel)parLevel).monsters.removeElement(parM);
            parLevel.elements.removeElement(parM);
        }
            
        
    }

    private void monsterBallInteraction(Ball parBall, Monster parM, NormalLevel parLevel){
        Direction locDir=parBall.getDirection();             
        if(parBall.getType()!=Ball.THROUGH_TYPE&&!parBall.beingHold){
                    Ball newBall=new Ball((Ball)parBall);
                    newBall.setPosition(newBall.getX()-(int)locDir.x, newBall.getY());
                    if(newBall.intersects(parM)){
                        parBall.changeDirection(new Direction(locDir.x,-locDir.y));
                        parBall.position.y+=locDir.y;
                    }
                    else{
                        parBall.changeDirection(new Direction(-locDir.x,locDir.y));
                        parBall.position.x+=locDir.x;
                    }
                    if(parM.getType()==Monster.BALL_EATER)
                        parLevel.lostBall(parBall);
                    else if(parM.getType()==Monster.BALL_FIRER)
                        parBall.setType(Ball.FIRE_TYPE);
                    else if(parM.getType()==Monster.BALL_SENDER){
                        parBall.beingHold=true;
                        parM.setCapturedBall(parBall);
                        parM.hasCapturedBall=true;
                    }
         }
    }

    public void bossMotion(MonsterLevel parLevel,BossMonster parBoss){
       Direction locDir=parBoss.getDirection();
       if(parBoss.getActualY()>150){
			parBoss.changeDirection();		
        }else 
	  if(parBoss.getActualX()<=0){
            parBoss.changeDirection();
		locDir.x=Math.abs(locDir.x);
            parBoss.position.x+=locDir.x;
        }else if(parBoss.getActualX()+parBoss.getWidth()>=parLevel.walls[2].getActualX()){
            parBoss.changeDirection();
            locDir.x=-Math.abs(locDir.x);
            parBoss.position.x+=locDir.x;
	  }else if(parBoss.getActualY()<=0){
            parBoss.changeDirection();
            locDir.y=Math.abs(locDir.y);
            parBoss.position.y+=locDir.y;
        }
    }
}