/*
The ball class is used to initialize ball objects that are movable and has different types.
The movement of these ball objects are managed by a ball manager object.

The main information pass is with the level class to which this class belongs.

Copyright(c) Group Seven.
Added to the OO-Ball in the first (1.0) version.
*/

import java.awt.*;
import java.awt.image.*;

public class Ball extends TypedObject implements Movable
{
    static final int BIG_SIZE=1, NORMAL_SIZE=0, TINY_SIZE=-1;
    static final int FIRE_TYPE=2, NORMAL_TYPE=0, THROUGH_TYPE=1, MAD_TYPE=3;
    static final int SIZE_OF_NORMAL_BALL=10;
    static final int SIZE_DIFFERENCE=5;
    static final String imageFile="Images/Ball.gif";
    private int size=SIZE_OF_NORMAL_BALL;
    private Direction directionOfBall;
    private Image wholeImage;
    private ImageProducer producer;
    public boolean stop=true,beingHold=true;
    
    Ball(int parX, int parY, int parDirX, int parDirY, Component parOwner,LevelTemplate parLevel){
        super(parX, parY, parOwner, parLevel);
        directionOfBall=new Direction(parDirX,parDirY);
        try{
            wholeImage=parOwner.getToolkit().getImage(imageFile);
        try {
            MediaTracker tracker = new MediaTracker(owner);
            tracker.addImage(wholeImage, 0);
            tracker.waitForID(0);
        }
        catch ( Exception e ) {}
        }
        catch(Exception e){
            System.out.println("Image file for ball object could not be found! [image/Ball.gif]");
        }
        wholeImage.flush();
        setType(NORMAL_TYPE);        
    }
    
    Ball(int parX, int parY,int parDirX, int parDirY, Component parOwner, int parSize, int parType,LevelTemplate parLevel){
        this(parX, parY, parDirX, parDirY, parOwner, parLevel);
        setSize(parSize);
        setType(parType);        
    }

    Ball(Ball parBall){
        this(parBall.getX(), parBall.getY(), (int)parBall.getDirection().x, (int)parBall.getDirection().y, parBall.owner,parBall.level);
        setSize(parBall.getSize());
        setType(parBall.getType());
        beingHold=parBall.beingHold;
    }
        
    
    //move the ball in the current direction
    public void move(){
        if(type==MAD_TYPE){
            double locRanX=directionOfBall.x;
            if(directionOfBall.x!=0)
                locRanX=Math.exp(directionOfBall.x)/directionOfBall.x;            
            double locRanY=directionOfBall.y*Math.exp(directionOfBall.y);
            directionOfBall.setDirection(locRanX,locRanY);            
        }
        position.x+=directionOfBall.x;
        position.y+=directionOfBall.y;
        level.handleAction(this);        
    }
    
    //change the direction of the ball
    public void changeDirection(Direction parDirection){
        directionOfBall.setDirection(parDirection.x, parDirection.y);
    }
      
    //return the current direction of the ball
    public Direction getDirection() {
        return directionOfBall;
    }
    
    //set the type of the ball to one of the available types
    //and get the image for the new ball type
    public void setType(int parType){
        CropImageFilter locCrop;
        locCrop=new CropImageFilter(0,0,20,20);
        type=parType;
        switch(parType){
            case FIRE_TYPE:
                locCrop=new CropImageFilter(20,0,20,20);
                break;
            case THROUGH_TYPE:
                locCrop=new CropImageFilter(40,0,20,20);
                break;
            case MAD_TYPE:
                locCrop=new CropImageFilter(60,0,20,20);
                break;
        }
        producer = new FilteredImageSource(wholeImage.getSource(), locCrop);
        imageOfObject = owner.createImage(producer);
        try {
            MediaTracker tracker = new MediaTracker(owner);
            tracker.addImage(imageOfObject, 0);
            tracker.waitForID(0);
        }
        catch ( Exception e ) {}
        
    }
    
    
    //set the size of the ball to one of the predefineds
    public void setSize(int parSize){
        size=parSize;
        if(parSize==BIG_SIZE){
            setHeight(SIZE_OF_NORMAL_BALL+SIZE_DIFFERENCE);
            setWidth(SIZE_OF_NORMAL_BALL+SIZE_DIFFERENCE);
        }
        else if(parSize==TINY_SIZE){
            setHeight(SIZE_OF_NORMAL_BALL-SIZE_DIFFERENCE);
            setWidth(SIZE_OF_NORMAL_BALL-SIZE_DIFFERENCE);
        }
        else{
            setHeight(SIZE_OF_NORMAL_BALL);
            setWidth(SIZE_OF_NORMAL_BALL);
        }
    }
    
    //return the size of the ball as one of the predefineds
    public int getSize(){
        return size;
    }
    
    
    //return the ball to its initial stage
    public void returnToInitialType(){
        setType(NORMAL_TYPE);
        setSize(NORMAL_SIZE);
    }

    //return whether or not the ball object intersects paddle object
    public boolean intersects(Paddle parPad){
	 return parPad.intersects(this);
    }
} 