/*
The BallManager controls the movement of the ball objects in a level. 
It is responsible from continues movement of those balls while it is not stopped.

The main information pass is with the level class to which this class belongs.

Copyright(c) Group Seven.
Added to the OO-Ball in the 1.2 version, modified in the 1.3 version.
*/

import java.util.*;

public class BallManager extends ManagerTemplate
{
    static final int FAST_SPEED=1, NORMAL_SPEED=0, SLOW_SPEED=-1;
    static final int SPEED_OF_NORMAL_BALL=20;
    static final int SPEED_DIFFERENCE=10;
    static final int MAX_WAIT=70;
    private int speed=NORMAL_SPEED;
    
    BallManager(LevelTemplate parLevel){
	super(parLevel);
	waitTime=MAX_WAIT - SPEED_OF_NORMAL_BALL;      
	Th=new Thread(this);
    }

    //starts running the thread to start the actions, movement of balls
    public void start(){
	  if(!Th.isAlive())
	       Th.start();
	  else
		 resume();
    }
    
    //set the speed of balls to one of the predefineds
    public void setSpeed(int parSpeed){
        speed=parSpeed;
        waitTime=MAX_WAIT-returnSpeed(parSpeed);
    }
    
    //return the speed of the balls as one of the predefineds
    public int getSpeed(){
        return speed;
    }

    //returns the actual speed of the balls given the spped in terms of a predefined type
    public int returnSpeed(int parSpeed){
        switch(parSpeed){
            case FAST_SPEED:
                return SPEED_OF_NORMAL_BALL + SPEED_DIFFERENCE;
            case SLOW_SPEED:
                return SPEED_OF_NORMAL_BALL - SPEED_DIFFERENCE;            
        }
        return SPEED_OF_NORMAL_BALL;
    }
    
    //runs the thread until it is stopped and moves the ball objects in specific time
    //periods
    public void run(){
        Ball nextB;
	  while(true){
            for (Enumeration e = level.balls.elements() ; e.hasMoreElements() ;) {
                nextB=((Ball)e.nextElement());
	          if(level.ownerGame.hasLevelStarted()&&!nextB.beingHold)
                    nextB.move();       
            }
            try{
                level.ownerGame.notifyObserver();
                Th.sleep(waitTime);                            
            }catch(Exception e){System.out.println("Error in ball control");}
         }
    }
}