/*
This class is used to initialize the bonus objects in a game.
The class is movable, is controlled by a manager and has different types.

The main information pass is with the level class to which this class belongs.

Copyright(c) Group Seven.
Added to the OO-Ball in the 1.1 version.
*/

import java.awt.*;
import java.awt.image.*;

public class Bonus extends TypedObject implements Movable
{
    private Image wholeImage;    
    private ImageProducer producer;    
    private static final int VERTICAL_DISPLACEMENT=5;
    public static final int WIDTH=Brick.WIDTH, HEIGHT=Brick.HEIGHT;
    private static final String imageFile="Images/Bonus.gif";
    public static final int FINISH_LEVEL=0, RESTART_LEVEL=1, DEATH=2, LIFE_PLUS_1=3,EXPLODE_EXPLOSIVES=4,
    HIT_ALL_BRICKS_ONCE=5, BRICKS_COME_DOWN=6, STOP_MONSTERS=7, START_MONSTERS=8, SLOW_MONSTERS=9,
    FAST_MONSTERS=10, KILL_MONSTERS=11, VERTICAL_MOTION_ENABLE=12, VERTICAL_MOTION_DISABLE=13, BONUS=14,
    FROZEN_PADDLE=15, FAST_BALL=16, SLOW_BALL=17, BIG_BALL=18, SMALL_BALL=19, FIRE_BALL=20, THROUGH_BALL=21,
    MAD_BALL=22, LASER_PADDLE=23, STICKY_PADDLE=24, SPLIT_PADDLE=25, SHORTER_PADDLE=26, LONGER_PADDLE=27,
    EIGHT_BALL=28, TWO_BALL=29;

    Bonus(int parX, int parY, int parType, Component parOwner,NormalLevel parLevel){
        super(parX, parY, parOwner, parLevel);  
        try{
            wholeImage=parOwner.getToolkit().getImage(imageFile);
        try {
            MediaTracker tracker = new MediaTracker(owner);
            tracker.addImage(wholeImage, 0);
            tracker.waitForID(0);
        }
        catch ( Exception e ) {}
        }
        catch(Exception e){
            System.out.println("Image file for bonus object could not be found! [image/Bonus.gif]");
        }
        setType(parType);
        setWidth(WIDTH);
        setHeight(HEIGHT);  
    }

    //sets the current type of the bonus to a predefined one
    public void setType(int parType){
        CropImageFilter locCrop;        
        type=parType;
        if(parType<10){
            locCrop=new CropImageFilter(0,15*parType,40,15);
        }
        else if(parType<20){
            locCrop=new CropImageFilter(40,15*(parType-10),40,15);
        }
        else{
            locCrop=new CropImageFilter(80,15*(parType-20),40,15);
        }
        producer = new FilteredImageSource(wholeImage.getSource(), locCrop);
        imageOfObject = owner.createImage(producer);
        try {
            MediaTracker tracker = new MediaTracker(owner);
            tracker.addImage(imageOfObject, 0);
            tracker.waitForID(0);
        }
        catch ( Exception e ) {}
    }

    //changes the current position of the bonus in the given direction
    public void move(){
        position.y+=VERTICAL_DISPLACEMENT;
        ((NormalLevel)level).handleAction(this);
    }

    //returns whether or not the bonus intersects paddle object
    public boolean intersects(Paddle parPad){
	 return parPad.intersects(this);
	}

}