/*
The BossMonster class is used for the boss monster object in MonsterLevel.
It can move on its own without using a manager for this operation.

The main information pass is with the level class to which this class belongs.

Copyright(c) Group Seven.
Added to the OO-Ball in the 1.2 version.
*/

import java.awt.*;
import java.util.*;

public class BossMonster extends ObjectTemplate implements Movable, Runnable
{
    private Thread Th;
    private static final int NO_OF_IMAGES=2;
    private Image images[]=new Image[NO_OF_IMAGES];
    private int nextImage=0;
    private static final String imageFile="Images/monsters/boss/";
    private static final int waitTime=100;
    private Direction dir=new Direction(1,0);
    private int strength=15;
    private Random r=new Random();    

    BossMonster(int parX, int parY, Component parOwner, MonsterLevel parLevel){
        super(parX,parY,parOwner, parLevel);
        dim=new Dimension(110,110);
        imageOfObject=getImages();
        Th=new Thread(this);
    }

    //starts the motion of the monster
    public void start(){
        if(Th.isAlive())
            resume();
        else
            Th.start();
    }

    //returns the current direction
    public Direction getDirection(){
	return dir;
    }

    //retrieves the images necessary for the animation from the image file
    protected Image getImages(){
        for(int i=0;i<NO_OF_IMAGES;i++){
            try{
                images[i]=owner.getToolkit().getImage(imageFile+"boss"+i+".gif");
            try {
                MediaTracker tracker = new MediaTracker(owner);
                tracker.addImage(images[i], 0);
                tracker.waitForID(0);
            }
            catch ( Exception e ) {}
            }
            catch(Exception e){
                System.out.println("Image file for boss monster object could not be found! [image/Bonus.gif]");
            }        
        }
        return images[0];
    }

    //stops the movement
    public void stop(){
        Th.stop();
    }

    //resumes the movement
    public void resume(){
        Th.resume();
    }

    //suspends the movement
    public void suspend(){
        Th.suspend();
    }

    //moves in the current direction
    public void move(){
	  position.x+=dir.x;
	  position.y+=dir.y;
        ((MonsterLevel)level).handleAction(this);
        nextImage=(nextImage+1)%NO_OF_IMAGES;        
        imageOfObject=images[nextImage];         
    }

    //supplies continuous movement of the monster
    public void run(){
        int noOfTimes=0;
        while(true){            
            move();
            try{
                if(noOfTimes==100||noOfTimes==50){
                    Th.sleep(500);
                    if(noOfTimes==50){
                        fire(Fire.SMALL_FIRE);    
                    }else{
                        noOfTimes=0;  
                        fire(Fire.BIG_FIRE);    
                    }
                    Th.sleep(100);                
                }
                noOfTimes++;
                level.ownerGame.notifyObserver();
                Th.sleep((long)waitTime);
            }
            catch(Exception e){
                System.out.println("Error in boss monster thread!");
            }
        }
    }   

    //fires Fire objects
    private void fire(int fireType){
        Fire newFire;
        if(fireType==Fire.BIG_FIRE){
            newFire=new Fire(getX()+getWidth()/2,getY()+getHeight(),Fire.BIG_FIRE,new Direction(0,1),owner,level);
            level.fires.addElement(newFire);
            level.elements.addElement(newFire);            
        }
        else{
            Direction dir[]={new Direction(0,1),new Direction(1,1),new Direction(1,0),
                          new Direction(-1,1),new Direction(-1,0)};
            for(int i=0; i<5; i++){
                newFire=new Fire(getX()+getWidth()/2,getY()+getHeight(),Fire.SMALL_FIRE,dir[i],owner,level);
                level.fires.addElement(newFire);
                level.elements.addElement(newFire);
            }
        }
    }

    //changes the current direction
    public void changeDirection(){
	  position.x-=dir.x;
	  position.y-=dir.y;
	  int dirX=r.nextInt();
	  int dirY=r.nextInt();
        if(dirX==dirY&&dirX==0){
		dirX=1;
   		dirY=1;
	  }
        dir=new Direction(dirX,dirY);
    }

    //the monster is hit by the ball
    public boolean hit(){
	  position.x-=dir.x;
	  position.y-=dir.y;
	  dir.x=-dir.x;
	  dir.y=-dir.y;
	  strength--;
	  level.ownerGame.addScore(250);
        if(strength==0){
	      level.ownerGame.addScore(2500);
            return true;
	  }
        else
            return false;
    }
}