/*
The utility class that is used to load/save games/levels/high scores.

Copyright(c) Group Seven.
Added to the OO-Ball in the 1.1 version.
Additional information is in API Reference in our web page.
*/

import java.io.*;
import java.util.*;

public final class FileManager
{
    static final String GAME_DIRECTORY="data/games/";
    static final String GAME_EXT=".gm";
    static final String LEVEL_DIRECTORY="data/levels/";
    static final String LEVEL_EXT=".lv";
    static final String HI_SCORE_FILE="data/scores.hi";
    
    public static final String[][] getHighScores(int parIndX, int parIndY){
        BufferedReader in;
	  String parList[][]=new String[parIndX][parIndY];
        String line;
        StringTokenizer tokenizer;
        try{
            in= new BufferedReader(new FileReader(HI_SCORE_FILE));
            for(int i=0;i<10;i++){
        	    parList[i][0]=""+(i+1);
                line=in.readLine();
                tokenizer=new StringTokenizer(line,",");
                for(int j=0;tokenizer.hasMoreTokens();j++)
                    parList[i][j+1]=tokenizer.nextToken();
            }
            in.close();
        }
        catch(Exception e){
            System.out.println("ERRF1: Error in reading file. High score file.");         
        }
        return parList;
    }
    public static final void setHighScores(String parList[][]){
		BufferedWriter out;
		String line;
	try{
		out=new BufferedWriter(new FileWriter(HI_SCORE_FILE));	      
		for(int i=0;i<10;i++){
			line="";
      	      for(int j=1;j<5;j++){	
			if(j!=1)
      			line=line+","+parList[i][j];
			else
				line=parList[i][j];
			}
			out.write(line,0,line.length());
			out.newLine();
		}
		out.close();
	}catch(Exception e){
            System.out.println("ERRF3: Error in writing file. High score file.");         
	}
		
    }	
    public static final void saveLevel(String parList[]){	
    	saveText(parList,LEVEL_DIRECTORY);
    }
    public static final void saveGame(String parList[]){	
    	saveText(parList,GAME_DIRECTORY);
    }
    public static final void saveText(String parList[],String parPath){
		BufferedWriter out;
		String line;
	try{
		out=new BufferedWriter(new FileWriter(parPath+parList[0]));	      
		for(int i=1;i<parList.length;i++){
			out.write(parList[i],0,parList[i].length());
			out.newLine();
		}
		out.close();
	}catch(Exception e){
            System.out.println("ERRF4: Error in writing file.");         
	}
		
    }
    public static final String[][] getGameList(){
        File directory=new File(GAME_DIRECTORY);
        String fileList[]=directory.list();
        String parList[][]=new String[fileList.length][3];
        int j=0;
        BufferedReader in;
                
        for(int i=0; i<fileList.length;i++)
            if(fileList[i].substring(fileList[i].length()-3,fileList[i].length()).equals(GAME_EXT)){
                parList[j][0]=fileList[i];
                try{
                    in= new BufferedReader(new FileReader(GAME_DIRECTORY+fileList[i]));
                    parList[j][1]=in.readLine();
                    parList[j][2]=in.readLine();
                    j++;
                }catch(Exception ex){}
            }
		String newList[][]=new String[j][3];
		for(int l=0;l<3;l++)
			for(int k=0;k<j;k++)
				newList[k][l]=parList[k][l];
        return newList;
        
    }
    public static final String[] getBrickList(){
        File directory=new File(Brick.DIRECTORY);
        String fileList[]=directory.list();
        String parList[]=new String[fileList.length];
        int j=0;
                
        for(int i=0; i<fileList.length;i++)
            if(fileList[i].substring(fileList[i].length()-4,fileList[i].length()).equals(".gif")){
                parList[j]=fileList[i];	
		    j++;
            }
		
		String newList[]=new String[j];
		for(int k=0;k<j;k++)
			newList[k]=parList[k];
        return newList;
    }
    public static final String[] getLevelList(){
        File directory=new File(LEVEL_DIRECTORY);
        String fileList[]=directory.list();
	  String parList[]=new String[fileList.length];        
	  BufferedReader in;
        int j=0;
        for(int i=0; i<fileList.length;i++)
            if(fileList[i].substring(fileList[i].length()-3,fileList[i].length()).equals(LEVEL_EXT)){
                parList[j]=fileList[i];
		    j++;
            }
		
		String newList[]=new String[j];
		for(int k=0;k<j;k++)
			newList[k]=parList[k];
        return newList;
    }
    public static final void getGame(Game parGame, String parFileName){
        BufferedReader in;
        String line;        
        try{
            in= new BufferedReader(new FileReader(GAME_DIRECTORY+parFileName));
            parGame.setName(in.readLine());
            parGame.setNoOfLevels(Integer.parseInt(in.readLine()));
            parGame.levels=new LevelTemplate[parGame.getNoOfLevels()];
            for(int i=0;i<parGame.getNoOfLevels();i++){
                parGame.levels[i]=getLevel(parGame,in.readLine());                
            }
            in.close();
        }
        catch(Exception e){
            System.out.println("ERRF2: Error in reading file. Game File.");
        }
    }
    public static final LevelTemplate getLevel(Game parGame, String parFileName){
        LevelTemplate parLevel;
        BufferedReader in;
        String line;
        StringTokenizer tokenizer;
        try{
            in= new BufferedReader(new FileReader(LEVEL_DIRECTORY+parFileName));
            if(in.readLine().equals("Normal")){
                parLevel=new NormalLevel(parGame.getOwner(),parGame);
                parLevel.setName(in.readLine());
                if(in.readLine().equals("true"))
                    ((NormalLevel)parLevel).setHasMonsters(true);
                    
                try{
                    int xCoor, yCoor;
                    String xCode, yCode;
                    Brick newBrick;
                    Random r=new Random();
                    try{
                        for(;;){
                            line=in.readLine();
                            tokenizer=new StringTokenizer(line,",");
                            xCode=tokenizer.nextToken();
                            xCoor=Integer.parseInt(xCode)*Brick.WIDTH;
                            yCode=tokenizer.nextToken();
                            yCoor=Integer.parseInt(yCode)*Brick.HEIGHT;
                            if(tokenizer.nextToken().equals("true")){
                                newBrick=new MovingBrick(LevelTemplate.position.x+xCoor+Wall.SIZE , LevelTemplate.position.y+yCoor+Wall.SIZE , Integer.parseInt(tokenizer.nextToken()), tokenizer.nextToken(),parGame.getOwner(), tokenizer.nextToken(), (NormalLevel)parLevel);
                                ((NormalLevel)parLevel).mvBricks.addElement((MovingBrick)newBrick);                                
                            }else{
                                newBrick=new Brick(LevelTemplate.position.x+xCoor+Wall.SIZE , LevelTemplate.position.y+yCoor+Wall.SIZE , Integer.parseInt(tokenizer.nextToken()), tokenizer.nextToken(),parGame.getOwner(), tokenizer.nextToken(), (NormalLevel)parLevel);
                                ((NormalLevel)parLevel).bricks.put(xCode+yCode,newBrick);
                            }
                            parLevel.elements.addElement(newBrick);
                            if(newBrick.hasBonus)
					newBrick.setBonus(new Bonus(newBrick.getX(),newBrick.getY(),(Math.abs(r.nextInt())%30),parGame.getOwner(), (NormalLevel)parLevel));
                            
                        }                           
                    }catch(Exception e){}
                }catch(Exception e){}
                ((NormalLevel)parLevel).mvManager=new MovingBrickManager((NormalLevel)parLevel);
                ((NormalLevel)parLevel).bnManager=new BonusManager((NormalLevel)parLevel);                
            }
            else{
                parLevel=new MonsterLevel(parGame.getOwner(),parGame);
                parLevel.setName(in.readLine());                
            }            
            in.close();
            parLevel.bManager=new BallManager(parLevel);
            parLevel.fManager=new FireManager(parLevel);
            parLevel.setFileName(parFileName);
            return parLevel;
        }
        catch(Exception e){}
        return null;
    }
    
}