/*
The main class that includes a sequence of levels 
and coordinate the necessary operations in a game.

Copyright(c) Group Seven.
Added to the OO-Ball in the 1.0 version.
Additional information is in API Reference in our web page.
*/

import java.awt.*;

public class Game
{
    private static final int BONUS_VALUE=1000;
    private int currentLevel=0;
    private ActionManager actMan;
    private GameFrame observer;
    private Component ownerComponent;
    private int noOfLevels;
    private String nameOfGame;
    private boolean startLevel=false;
    private int noOfLives=5;
    private int score=0;
    protected LevelTemplate levels[];
    
    Game(Component parOwner, String parGameFile){
        ownerComponent=parOwner;
        FileManager.getGame(this, parGameFile);
        actMan=new ActionManager(this);
    }
    public void addBonus(){
        addScore(BONUS_VALUE);
    }
    public void addScore(int parAdd){
        score+=parAdd;
    }
    public String getName(){
	return nameOfGame;
    }
    public void setName(String parName){
	nameOfGame=parName;
    }
    public int getScore(){
	return score;
    }
    public void setNoOfLives(int parNo){
	noOfLives=parNo;
    }
    public int getNoOfLives(){
	return noOfLives;
    }
    public void setNoOfLevels(int parNo){
	noOfLevels=parNo;
    }
    public int getNoOfLevels(){
	return noOfLevels;
    }
    public LevelTemplate getCurrentLevel(){
  	return levels[currentLevel];
    }
    public int getCurrentLevelNo(){
  	return currentLevel;
    }
    public void initialize(){
        levels[currentLevel].initialize();
    }
    public void addObserver(GameFrame parO){
        observer=parO;
    }
    public GameFrame getObserver(){
	return observer;
    }
    public Component getOwner(){
	return ownerComponent;
    }
    public ActionManager getActionManager(){
	return actMan;
    }
    public void levelStarted(boolean parStarted){
		startLevel=parStarted;
    }
    public boolean hasLevelStarted(){
		return startLevel;
    }
   public void notifyObserver(){
        observer.repaint();
    }
    public void death(){
        noOfLives--;
        startLevel=false;            
        if(noOfLives==0)
            lostGame();
        else{
            levels[currentLevel].death();
        }	  
    }
    public void lostGame(){
	  observer.gameFinished(false);	  
    }
    public void restartLevel(){
        levels[currentLevel].finish();
        levels[currentLevel]=FileManager.getLevel(this, levels[currentLevel].getFileName());
        startLevel=false;
        initialize();
        observer.repaint();
    }
    public void incLife(){
        noOfLives++;
    }
    public void finishLevel(){
        startLevel=false;
        levels[currentLevel].finish();        
        if(currentLevel==noOfLevels-1){
            finishGame();
        }
        else{
		currentLevel++;
            initialize();
            observer.repaint();
        }
    }
    private void finishGame(){
	  observer.gameFinished(true);	  
    }
    public void startAction(){
        if(levels[currentLevel].paddle.getType()==Paddle.LASER)
            actMan.fireLaser((NormalLevel)levels[currentLevel]);
        actMan.releaseBall(levels[currentLevel]);     
    }
    public void movePaddle(Point parPosition){
        actMan.movePaddle(levels[currentLevel],parPosition);        
    }
    public void movePaddle(int parRight, int parDown){
        actMan.movePaddle(levels[currentLevel],parRight,parDown);        
    }
    public void draw(Graphics g){
        levels[currentLevel].draw(g);
    }
}