/*
The dialog used to form new custom games.

Copyright(c) Group Seven.
Added to the OO-Ball in the 1.3 version.
Additional information is in API Reference in our web page.
*/

import java.awt.*;

public class GameEditorDialog extends Dialog
{
	private String levelList[];
	public GameEditorDialog(Frame parent)
	{
		super(parent);

		setLayout(null);
		setBackground(java.awt.Color.black);
		setForeground(java.awt.Color.green);
		setSize(430,270);
		setVisible(false);
		levelList=FileManager.getLevelList();
		availList.setForeground(java.awt.Color.black);
		add(availList);
		availList.setBounds(24,60,156,156);
		for(int i=0;i<levelList.length;i++)
			availList.add(levelList[i]);
		gameList.setForeground(java.awt.Color.black);
		add(gameList);
		gameList.setBounds(240,60,156,156);
		label1.setText("Available Levels:");
		label1.setAlignment(java.awt.Label.CENTER);
		add(label1);
		label1.setBounds(24,24,155,24);
		label2.setText("Levels In Game:");
		label2.setAlignment(java.awt.Label.CENTER);
		add(label2);
		label2.setBounds(240,24,155,24);
		addButton.setLabel(">>");
		add(addButton);
		addButton.setBackground(java.awt.Color.lightGray);
		addButton.setForeground(java.awt.Color.black);
		addButton.setBounds(192,96,38,18);
		removeButton.setLabel("<<");
		add(removeButton);
		removeButton.setBackground(java.awt.Color.lightGray);
		removeButton.setForeground(java.awt.Color.black);
		removeButton.setBounds(192,156,38,18);
		OKButton.setLabel("OK");
		add(OKButton);
		OKButton.setBackground(java.awt.Color.lightGray);
		OKButton.setForeground(java.awt.Color.black);
		OKButton.setBounds(24,228,75,24);
		cancelButton.setLabel("Cancel");
		add(cancelButton);
		cancelButton.setBackground(java.awt.Color.lightGray);
		cancelButton.setForeground(java.awt.Color.black);
		cancelButton.setBounds(108,228,75,24);
		editorButton.setLabel("Level Editor");
		add(editorButton);
		editorButton.setBackground(java.awt.Color.lightGray);
		editorButton.setForeground(java.awt.Color.black);
		editorButton.setBounds(276,228,90,24);
		setTitle("Game Editor");
		setResizable(false);
		//}}

		//{{REGISTER_LISTENERS
		SymWindow aSymWindow = new SymWindow();
		this.addWindowListener(aSymWindow);
		SymAction lSymAction = new SymAction();
		editorButton.addActionListener(lSymAction);
		cancelButton.addActionListener(lSymAction);
		OKButton.addActionListener(lSymAction);
		addButton.addActionListener(lSymAction);
		removeButton.addActionListener(lSymAction);
		//}}
	}
	
	public GameEditorDialog(Frame parent, boolean modal)
	{
		this(parent);
        setModal(modal);
	}
	
	public void addNotify()
	{
  	    // Record the size of the window prior to calling parents addNotify.
	    Dimension d = getSize();

		super.addNotify();

		if (fComponentsAdjusted)
			return;

		// Adjust components according to the insets
		Insets insets = getInsets();
		setSize(insets.left + insets.right + d.width, insets.top + insets.bottom + d.height);
		Component components[] = getComponents();
		for (int i = 0; i < components.length; i++)
		{
			Point p = components[i].getLocation();
			p.translate(insets.left, insets.top);
			components[i].setLocation(p);
		}
		fComponentsAdjusted = true;
	}

    // Used for addNotify check.
	boolean fComponentsAdjusted = false;


	public GameEditorDialog(Frame parent, String title, boolean modal)
	{
		this(parent, modal);
		setTitle(title);
	}

    /**
     * Shows or hides the component depending on the boolean flag b.
     * @param b  if true, show the component; otherwise, hide the component.
     * @see java.awt.Component#isVisible
     */
    public void setVisible(boolean b)
	{
		if(b)
		{
			Rectangle bounds = getParent().getBounds();
			Rectangle abounds = getBounds();
	
			setLocation(bounds.x + (bounds.width - abounds.width)/ 2,
				 bounds.y + (bounds.height - abounds.height)/2);
		}
		super.setVisible(b);
	}

	//{{DECLARE_CONTROLS
	List availList = new List(0);
	List gameList = new List(0);
	Label label1 = new Label();
	Label label2 = new Label();
	Button addButton = new Button();
	Button removeButton = new Button();
	Button OKButton = new Button();
	Button cancelButton = new Button();
	Button editorButton = new Button();
	//}}

	class SymWindow extends java.awt.event.WindowAdapter
	{
		public void windowClosing(java.awt.event.WindowEvent event)
		{
			Object object = event.getSource();
			if (object == this)
				GameEditor_WindowClosing(event);
		}
	}
	
	void GameEditor_WindowClosing(java.awt.event.WindowEvent event)
	{
		setVisible(false);
	}

	class SymAction implements java.awt.event.ActionListener
	{
		public void actionPerformed(java.awt.event.ActionEvent event)
		{
			Object object = event.getSource();
			if (object == editorButton)
				editorButton_ActionPerformed(event);
			else if (object == cancelButton)
				cancelButton_ActionPerformed(event);
			else if (object == OKButton)
				OKButton_ActionPerformed(event);
			else if (object == addButton)
				addButton_ActionPerformed(event);
			else if (object == removeButton)
				removeButton_ActionPerformed(event);
		}
	}

	protected void editorButton_ActionPerformed(java.awt.event.ActionEvent event)
	{
			 
		editorButton_ActionPerformed_Interaction1(event);
	}

	protected void editorButton_ActionPerformed_Interaction1(java.awt.event.ActionEvent event)
	{
		try {
			setVisible(false);
		        LevelEditorFrame editG=new LevelEditorFrame();
		        editG.show();	
		} catch (Exception e) {
		}
	}

	protected void cancelButton_ActionPerformed(java.awt.event.ActionEvent event)
	{
		// to do: code goes here.
			 
		cancelButton_ActionPerformed_Interaction1(event);
	}

	protected void cancelButton_ActionPerformed_Interaction1(java.awt.event.ActionEvent event)
	{
		try {
			// GameEditor Hide the GameEditor
			this.setVisible(false);
		} catch (Exception e) {
		}
	}

	protected void OKButton_ActionPerformed(java.awt.event.ActionEvent event)
	{
			 
		OKButton_ActionPerformed_Interaction1(event);
	}

	protected void OKButton_ActionPerformed_Interaction1(java.awt.event.ActionEvent event)
	{
		try {
			SaveLevelDialog slName=new SaveLevelDialog((Frame)getParent());
			slName.setTitle("Save Game");
			slName.fileNameLabel.setText("Game File:");
			slName.nameLabel.setText("Game Name:");
			slName.show();
			if(slName.isCanceled())
				return;
			String strList[]=new String[3+gameList.countItems()];
			strList[0]=slName.getFileName()+".gm";
			strList[1]=slName.getLevelName();
			strList[2]=""+gameList.countItems();
			for(int i=0;i<strList.length-3;i++)
				strList[3+i]=gameList.getItem(i);
			FileManager.saveGame(strList);
			this.setVisible(false);
		} catch (Exception e) {
		System.out.println("ops");
		}
	}

	protected void addButton_ActionPerformed(java.awt.event.ActionEvent event)
	{
			gameList.add(availList.getSelectedItem());
			 
	}

	protected void removeButton_ActionPerformed(java.awt.event.ActionEvent event)
	{
			 
		removeButton_ActionPerformed_Interaction1(event);
	}

	protected void removeButton_ActionPerformed_Interaction1(java.awt.event.ActionEvent event)
	{
		try {
			gameList.remove(gameList.getSelectedIndex());
		} catch (Exception e) {
		}
	}
}
