/*
The base class for levels.

Copyright(c) Group Seven.
Added to the OO-Ball in the 1.0 version.
Additional information is in API Reference in our web page.
*/

import java.util.*;
import java.awt.*;

public abstract class LevelTemplate
{
    protected Wall walls[]=new Wall[3];
    protected Paddle paddle;
    protected Game ownerGame;
    protected Component ownerComponent;
    static protected Point position=new Point(10,30);
    protected Vector balls=new Vector();
    static protected Dimension dim=new Dimension(450,400);
    protected Vector fires=new Vector();
    protected Vector elements=new Vector();
    private Graphics offGraph;
    private Image offScreen,bckGr;
    private Color bckColor=Color.black;
    private String nameOfLevel;
    private String fileName;
    protected FireManager fManager;
    protected BallManager bManager;
    protected boolean finished=false;
    
    LevelTemplate(Component parOwnerComponent, Game parOwnerGame){
        ownerComponent=parOwnerComponent;
        ownerGame=parOwnerGame;        
	  bckColor=((GreetingFrame)ownerComponent).getBckColor();
	  bckGr=((GreetingFrame)ownerComponent).getBckImage();
        paddle=new Paddle(position.x+dim.width/2-Paddle.NORMAL_SIZE/2,position.y+dim.height-Paddle.HEIGHT/2, parOwnerComponent, this);
        elements.addElement(paddle);        
        elements.addElement(walls[1]=new Wall(position.x, position.y, 0 , dim.width+Wall.SIZE, parOwnerComponent,this));
        elements.addElement(walls[0]=new Wall(position.x, position.y+Wall.SIZE, -1 , dim.height-Wall.SIZE, parOwnerComponent,this));
        elements.addElement(walls[2]=new Wall(position.x+dim.width, position.y+Wall.SIZE, 1 , dim.height-Wall.SIZE, parOwnerComponent,this));    
        
        Ball aBall=new Ball(paddle.getX()+paddle.getWidth()/2-5,paddle.getY()-10,0,  -1, parOwnerComponent,this);
        balls.addElement(aBall);        
        elements.addElement(aBall);	
    }
 
    public void initialize(){
        offScreen=ownerComponent.createImage(dim.width+200,dim.height+50);
        offGraph=offScreen.getGraphics();
        ownerComponent.repaint();
    }
    public String getFileName(){
	return fileName;
    }
    public void setFileName(String parFileName){
	fileName=parFileName;
    }
    public String getName(){
	return nameOfLevel;
    }
    public void setName(String parName){
	nameOfLevel=parName;
    }
    public void draw(Graphics g){
        Enumeration e;
        offGraph.setColor(bckColor);
        offGraph.fillRect(0, 0, dim.width+200,dim.height+50);
        if(bckGr!=null)
            offGraph.drawImage(bckGr,position.x,position.y,dim.width,dim.height,ownerComponent);
        for (e = elements.elements() ; e.hasMoreElements() ;) {
           ((ObjectTemplate)e.nextElement()).draw(offGraph);             
        }
	  offGraph.setColor(Color.green);
        offGraph.drawString("Score:", 490,250);
	  offGraph.drawString("Lives:", 490,300);
	  offGraph.drawString("Level:", 490,150);
	  offGraph.drawString("No Of Levels:", 490,200);
	  offGraph.drawString("Name Of Game:", 490,50);
	  offGraph.drawString("Name Of Level:", 490,100);
	  offGraph.setColor(Color.white);
	  offGraph.drawString(""+ownerGame.getScore(), 490,270);
	  offGraph.drawString(""+ownerGame.getNoOfLives(), 490,320);
	  offGraph.drawString(""+(ownerGame.getCurrentLevelNo()+1), 490,170);
	  offGraph.drawString(""+ownerGame.getNoOfLevels(), 490,220);
	  offGraph.drawString(""+ownerGame.getName(), 490,70);
	  offGraph.drawString(""+ownerGame.getCurrentLevel().getName(), 490,120);
	  offGraph.setColor(Color.red);
	  offGraph.drawString("Copyright(c)", 490,400);
	  offGraph.drawString("Group Seven", 490,420);
        g.drawImage(offScreen,0,0,ownerComponent);        
    }
    public void lostBall(Ball parBall){
        balls.removeElement(parBall);
        elements.removeElement(parBall);
        if(balls.isEmpty()){
	     ownerGame.death();
	  }            
    }
    public void finish(){
	  paddle.finalize();
        fires.removeAllElements();
        balls.removeAllElements();
        elements.removeAllElements();            
	  finished=true;        
	  Thread waitTh=new Thread();
	  waitTh.start();
	  Graphics locG=ownerGame.getObserver().getGraphics();
	  for(int i=0; i<20; i++){
		  locG.setColor(Color.red);
		  locG.drawString("LEVEL FINISHED", 250,220);		  
        	  try{ waitTh.sleep(100);}catch(Exception ex){}
	  }
	 waitTh.stop();

    }
    protected void death(){
	  Enumeration e;
        for (e = fires.elements() ; e.hasMoreElements() ;){
                Fire nextFire=((Fire)e.nextElement());
                elements.removeElement(nextFire);
        }
        fires.removeAllElements();
        for (e = balls.elements() ; e.hasMoreElements() ;){
                Ball nextBall=((Ball)e.nextElement());
                elements.removeElement(nextBall);                
        }
        balls.removeAllElements();
        paddle.finalize();
        elements.removeElement(paddle);
        paddle=new Paddle(position.x+dim.width/2-Paddle.NORMAL_SIZE/2,position.y+dim.height-Paddle.HEIGHT/2, ownerComponent, this);
        elements.addElement(paddle);        
        Ball aBall=new Ball(paddle.getX()+paddle.getWidth()/2-5,paddle.getY()-10,0,  -1, ownerComponent,this);
        balls.addElement(aBall);    
        elements.addElement(aBall);                
	  Thread waitTh=new Thread();
	  waitTh.start();
	  Graphics locG=ownerGame.getObserver().getGraphics();
	  for(int i=0; i<20; i++){
		  locG.setColor(Color.red);
		  locG.drawString("DEATH", 250,240);		  
        	  try{ waitTh.sleep(100);}catch(Exception ex){}
	  }
	 waitTh.stop();
  
    }

    abstract void start();
    abstract void handleAction(Ball parB);
    abstract void handleAction(Fire parF);
       
}