/*
The monster class which implements the movable monsters in the game which has different types.

Copyright(c) Group Seven.
Added to the OO-Ball in the 1.1 version.
Additional information is in API Reference in our web page.
*/

import java.awt.*;

public class Monster extends TypedObject implements Movable
{
    private Direction direction;
    private Image images[];
    private int currentImage=0, currentDirection=0;
    static final int BALL_EATER=0, HARMLESS=1, BALL_SENDER=2, BALL_FIRER=3;
    static final int MONSTER_SCORE=1000;
    static final String imageFiles="Images/monsters/";
    protected boolean hasCapturedBall=false;
    private Ball capturedBall;
    private int noOfMoves=0;
    
    Monster(int parX, int parY, int parType, Component parOwner, NormalLevel parLevel){
        super(parX,parY,parOwner, parLevel);
        dim=new Dimension(30,30);
        setType(parType);                                    
        imageOfObject=images[0];
    }
    
    public void move(){
        if(currentDirection==0)
            position.x+=10;
        else if(currentDirection==1)
            position.y-=10;
        if(currentDirection==2)
            position.x-=10;
        if(currentDirection==3)
            position.y+=10;
        if(hasCapturedBall){
            capturedBall.setX(position.x+dim.width/2-capturedBall.dim.width/2);
            capturedBall.setY(position.y+dim.height);
            level.handleAction(capturedBall);
        }
        ((NormalLevel)level).handleAction(this);
        currentImage=(currentImage+1)%images.length;        
        imageOfObject=images[currentImage]; 
        if(type==BALL_SENDER && hasCapturedBall){
		noOfMoves++;
            if(noOfMoves==100){
                noOfMoves=0;
                terminate();                                
            }
        }
    }

    public void setType(int parType){
        type=parType;
        switch(parType){
            case BALL_EATER:
                images=new Image[10];
                dim=new Dimension(50,50);
                for(int i=0;i<10;i++){
                    try{
                        images[i]=owner.getToolkit().getImage(imageFiles+"ballEater"+i+".gif");
                        try {
                            MediaTracker tracker = new MediaTracker(owner);                            
                            tracker.addImage(images[i], 0);
                            tracker.waitForID(0);
                        }
                        catch ( Exception e ) {}
                    }
                    catch(Exception e){
                        System.out.println("Image file for monster object could not be found! [image/Ball.gif]");
                    }
                }
                break;
            case BALL_FIRER:
                images=new Image[2];
                for(int i=0;i<2;i++){
                    try{
                        images[i]=owner.getToolkit().getImage(imageFiles+"ballFirer"+i+".gif");
                        try {
                            MediaTracker tracker = new MediaTracker(owner);                            
                            tracker.addImage(images[i], 0);
                            tracker.waitForID(0);
                        }
                        catch ( Exception e ) {}
                    }
                    catch(Exception e){
                        System.out.println("Image file for monster object could not be found! [image/Ball.gif]");
                    }
                }
                break;
            case BALL_SENDER:
                images=new Image[8];
                for(int i=0;i<8;i++){
                    try{
                        images[i]=owner.getToolkit().getImage(imageFiles+"ballSender"+i+".gif");
                        try {
                            MediaTracker tracker = new MediaTracker(owner);                            
                            tracker.addImage(images[i], 0);
                            tracker.waitForID(0);
                        }
                        catch ( Exception e ) {}
                    }
                    catch(Exception e){
                        System.out.println("Image file for monster object could not be found! [image/Ball.gif]");
                    }
                }
                break;
            default:
                images=new Image[2];
                for(int i=0;i<2;i++){
                    try{
                        images[i]=owner.getToolkit().getImage(imageFiles+"harmless"+i+".gif");
                        try {
                            MediaTracker tracker = new MediaTracker(owner);                            
                            tracker.addImage(images[i], 0);
                            tracker.waitForID(0);
                        }
                        catch ( Exception e ) {}                        
                    }
                    catch(Exception e){
                        System.out.println("Image file for monster object could not be found! [image/Ball.gif]");
                    }
                }
                break;
        }
    }
    public void nextDirection(){
        if(currentDirection==0)
            position.x-=10;
        else if(currentDirection==1)
            position.y+=10;
        if(currentDirection==2)
            position.x+=10;
        if(currentDirection==3)
            position.y-=10;
        currentDirection=((currentDirection+1)%4);        
    }
    public void terminate(){
        ((NormalLevel)level).monsters.removeElement(this);
        level.elements.removeElement(this);
        level.ownerGame.addScore(MONSTER_SCORE);        
        if(hasCapturedBall){
            capturedBall.changeDirection(new Direction(0,1));
            capturedBall.move();
            capturedBall.beingHold=false;
        }
        super.finalize();   
    }
    public boolean intersects(Paddle parPad){
	 return parPad.intersects(this);
    }
    public Ball getCapturedBall(){
		return capturedBall;
    }
    public void setCapturedBall(Ball parB){
		capturedBall=parB;
    }

}