/*
The normal level with bricks and monsters to be terminated.

Copyright(c) Group Seven.
Added to the OO-Ball in the 1.0 version.
Additional information is in API Reference in our web page.
*/

import java.util.*;
import java.awt.*;

public class NormalLevel extends LevelTemplate
{
    protected Vector mvBricks=new Vector();
    protected Vector monsters=new Vector();
    protected Vector bonuses=new Vector();
    protected Hashtable bricks=new Hashtable();
    private boolean hasMonsters=false;
    static final int MONSTER_Y=250,MONSTER_X=50;
    private boolean movingMonsters=true, bricksComeDown=false;
    protected MonsterManager mnManager;
    protected MovingBrickManager mvManager;
    protected BonusManager bnManager;
    private int i=0;
    private int currentMonsterType=Monster.HARMLESS;
    
    NormalLevel(Component parOwnerComponent, Game parOwnerGame){
        super(parOwnerComponent, parOwnerGame);        
    }    
    public void addMonster(){
        hasMonsters=true;
        if(currentMonsterType==Monster.BALL_EATER){
            i++;
            if(i==5){
                i=0;
            }else{currentMonsterType=(currentMonsterType+1)%4;}
        }
        Monster aMonster=new Monster(MONSTER_X+(int)(300*Math.random()),MONSTER_Y,currentMonsterType ,ownerComponent,this);
        for (Enumeration e = monsters.elements() ; e.hasMoreElements() ;){
                if(((Monster)e.nextElement()).intersects(aMonster)){
                    aMonster=new Monster(MONSTER_X+(int)(300*Math.random()),MONSTER_Y,currentMonsterType ,ownerComponent,this);
                    e=monsters.elements();
                }
        }
        monsters.addElement(aMonster);
        elements.addElement(aMonster);       
        currentMonsterType=(currentMonsterType+1)%4;        
    }

    public boolean areMonstersMoving(){
	return movingMonsters;
    }
    public void setMonstersMoving(boolean parMov){
	movingMonsters=parMov;
    }
    public boolean hasMonsters(){
	return hasMonsters;
    }
    public void setHasMonsters(boolean parHas){
	hasMonsters=parHas;
    }
    public boolean areBricksComingDown(){
	return bricksComeDown;
    }
    public void evalBonus(Bonus parBonus){
        ownerGame.addScore(250);
        switch(parBonus.type){    
        case(Bonus.FINISH_LEVEL):
            ownerGame.finishLevel();  
            break;
        case(Bonus.RESTART_LEVEL):
            ownerGame.restartLevel();
            break;
        case(Bonus.DEATH):
            ownerGame.death();
            break;
        case(Bonus.LIFE_PLUS_1):
            ownerGame.incLife();
            break;
        case(Bonus.EXPLODE_EXPLOSIVES):
            explodeBricks();
            break;
        case(Bonus.HIT_ALL_BRICKS_ONCE):
            hitAllBricks();
            break;
        case(Bonus.BRICKS_COME_DOWN):
            bricksComeDown=true;
            break;
        case(Bonus.STOP_MONSTERS):
            if(hasMonsters)
                stopMonsters();
            break;
        case(Bonus.START_MONSTERS):
            if(hasMonsters)
                startMonsters();        
            break;
        case(Bonus.SLOW_MONSTERS):
            if(hasMonsters)
                slowMonsters();
            break;
        case(Bonus.FAST_MONSTERS):
            if(hasMonsters)
                fastMonsters();
            break;
        case(Bonus.KILL_MONSTERS):
            if(hasMonsters)
                killMonsters();
            break;
        case(Bonus.VERTICAL_MOTION_ENABLE):
            paddle.verticalMotionEnable(true);
            break;
        case(Bonus.VERTICAL_MOTION_DISABLE):
            paddle.verticalMotionEnable(false);
            break;
        case(Bonus.BONUS):
            ownerGame.addBonus();
            break;
        case(Bonus.FROZEN_PADDLE):
            paddle.freeze();
            break;
        case(Bonus.FAST_BALL):
            changeBallSpeed(BallManager.FAST_SPEED);
            break;
        case(Bonus.SLOW_BALL):
            changeBallSpeed(BallManager.SLOW_SPEED);
            break;
        case(Bonus.BIG_BALL):
            changeBallSize(Ball.BIG_SIZE);
            break;
        case(Bonus.SMALL_BALL):
            changeBallSize(Ball.TINY_SIZE);
            break;
        case(Bonus.FIRE_BALL):
            changeBallType(Ball.FIRE_TYPE);
            break;
        case(Bonus.THROUGH_BALL):
            changeBallType(Ball.THROUGH_TYPE);
            break;
        case(Bonus.MAD_BALL):
            changeBallType(Ball.MAD_TYPE);
            break;
        case(Bonus.LASER_PADDLE):
            setPaddleType(Paddle.LASER);
            break;
        case(Bonus.STICKY_PADDLE):
            setPaddleType(Paddle.HOLD);
            break;
        case(Bonus.SPLIT_PADDLE):
            setPaddleType(Paddle.SPLIT);
		ownerGame.getActionManager().movePaddle(this,0,0);
            break;
        case(Bonus.SHORTER_PADDLE):
            paddle.shorten();
            break;
        case(Bonus.LONGER_PADDLE):
            paddle.enlarge();
            break;
        case(Bonus.EIGHT_BALL):
            eightBalls();
            break;
        case(Bonus.TWO_BALL):
            twoBalls();
            break;
        }
    }
    public void handleAction(Bonus parB){
        ownerGame.getActionManager().bonusMotion(this,parB);
    }
    public void handleAction(Monster parM){
        ownerGame.getActionManager().monsterMotion(this,parM);        
    }
    public void handleAction(MovingBrick parMov){
        ownerGame.getActionManager().brickMotion(this,parMov);
    }
    private void setPaddleType(int parType){
	if(paddle.type==Paddle.HOLD)	
       	ownerGame.getActionManager().releaseBall(this); 	
	paddle.setType(parType);

    }
    private void hitAllBricks(){
        Enumeration e;
        Brick nextBrick;
        for (e = bricks.elements() ; e.hasMoreElements() ;){
                nextBrick=((Brick)e.nextElement());
                if(nextBrick.hit(1))
                    nextBrick.terminate();
        }
        for (e = mvBricks.elements() ; e.hasMoreElements() ;){
                nextBrick=((Brick)e.nextElement());
                if(nextBrick.hit(1))
                    nextBrick.terminate();
        }
    }
    private void changeBallSpeed(int parSpeedType){
        bManager.setSpeed(parSpeedType);    
    }
    private void changeBallSize(int parSizeType){
        for (Enumeration e = balls.elements() ; e.hasMoreElements() ;)
                ((Ball)e.nextElement()).setSize(parSizeType);
    
    }
    private void changeBallType(int parType){
        for (Enumeration e = balls.elements() ; e.hasMoreElements() ;)
                ((Ball)e.nextElement()).setType(parType);
    
    }
    private void explodeBricks(){
        Enumeration e;
        Brick nextBrick;
        for (e = bricks.elements() ; e.hasMoreElements() ;){
                nextBrick=((Brick)e.nextElement());
                if(nextBrick.type==Brick.EXPLOSIVE)
                    nextBrick.terminate();
        }
        for (e = mvBricks.elements() ; e.hasMoreElements() ;){
                nextBrick=((Brick)e.nextElement());
                if(nextBrick.type==Brick.EXPLOSIVE)
                    nextBrick.terminate();
                    
        }
    }
    public void death(){
        movingMonsters=false;
        if(hasMonsters)
	    mnManager.stop();
        mvManager.stop();
	  Enumeration e;
        for (e = bonuses.elements() ; e.hasMoreElements() ;){
                Bonus nextBonus=((Bonus)e.nextElement());
                elements.removeElement(nextBonus);
        }
        bonuses.removeAllElements();
        super.death();
	  ownerGame.notifyObserver();
    }
    public void finish(){
        bricks.clear();
        movingMonsters=false;        
	  if(hasMonsters)
	        mnManager.stop();
        mvBricks.removeAllElements();
        monsters.removeAllElements();
        bonuses.removeAllElements();
        super.finish();
	  ownerGame.notifyObserver();
    }
    public void checkFinished(){
        if(finished)
            return;
        Enumeration e = mvBricks.elements() ;
        boolean notFinished=false;
        while (e.hasMoreElements()&& !notFinished){
            if(((MovingBrick)e.nextElement()).type!=Brick.UNBREAKABLE)
                notFinished=true;
        }
        e = bricks.elements() ;
        while (e.hasMoreElements()&& !notFinished){
            if(((Brick)e.nextElement()).type!=Brick.UNBREAKABLE)
                notFinished=true;
        }
        if(!notFinished){
            ownerGame.finishLevel();  
        }
    }
    public void start(){
        mvManager.start();
	  if(hasMonsters)
            initiateMonsters();
	  bManager.start();
        fManager.start();
    }
    public void stopMonsters(){
        Enumeration e;
        Monster nextM;
        if(movingMonsters){
            movingMonsters=false;
            mnManager.stop();
        for (e = monsters.elements() ; e.hasMoreElements() ;){
                nextM=((Monster)e.nextElement());            
                if(nextM.type==Monster.BALL_SENDER&&nextM.hasCapturedBall){
                    nextM.getCapturedBall().changeDirection(new Direction(0,1));
                    nextM.getCapturedBall().beingHold=false;
                }
                    
        }
        }
    }
    public void startMonsters(){
        Enumeration e;
        if(!movingMonsters){
            movingMonsters=true;
            mnManager.start();
        }
    }
    public void slowMonsters(){
        mnManager.setSpeed(MonsterManager.SLOW);    
    }
    public void fastMonsters(){
        mnManager.setSpeed(MonsterManager.FAST);        
    }
    public void killMonsters(){
        Enumeration e;
        for (e = monsters.elements() ; e.hasMoreElements() ;)
                ((Monster)e.nextElement()).terminate();                       
    
    }
    public void initiateMonsters(){
        mnManager=new MonsterManager(this);
        mnManager.start();
    }
    public void comeDownBricks(){
        Enumeration e, eM;
        String key, newKey;
        int nextKey;
        Brick nextBrick;        
        Monster nextM;
        MovingBrick nextMvB;
        for (e = bricks.keys() ; e.hasMoreElements() ;){
             key=(String)e.nextElement();
             nextKey=Integer.parseInt(key)+1;
             newKey="0000"+(nextKey);
             newKey=newKey.substring(newKey.length()-4, newKey.length());
 		 if((Brick)bricks.get(newKey)==null){
	             nextBrick=(Brick)bricks.remove(key);             
      	       nextBrick.setY(nextBrick.getY()+Brick.HEIGHT);
            	 if(nextBrick.getY()>MONSTER_Y-50)
            	    bricksComeDown=false;
            	 bricks.put(newKey, nextBrick);
            	 nextBrick.bonusOfBrick.setY(nextBrick.getY());
            	 for (eM = monsters.elements() ; eM.hasMoreElements() ;){
            	    nextM=((Monster)eM.nextElement());       
            	    if(nextBrick.intersects(nextM))
            	        nextM.terminate();
            	 }
            	 for (eM = mvBricks.elements() ; eM.hasMoreElements() ;){
            	    nextMvB=((MovingBrick)eM.nextElement());       
            	    if(nextBrick.intersects(nextMvB))
            	        nextMvB.terminate();
            	 }
		}
        }
    }
    private void eightBalls(){
        Ball newBall;
        Ball originalBall=(Ball)balls.firstElement();
        Direction dir[]={new Direction(0,1),new Direction(1,1),new Direction(1,0),
                          new Direction(0,-1),new Direction(1,-1),new Direction(-1,0),
                          new Direction(-1,-1)};
        for(int i=0; i<7; i++){
            newBall=new Ball((Ball)originalBall);
            newBall.changeDirection(new Direction(dir[i].x,dir[i].y));
            newBall.setPosition(new Point(originalBall.getPosition()));
            balls.addElement(newBall);
            elements.addElement(newBall);
        }
    }
    private void twoBalls(){
        Ball newBall;
        Ball originalBall=(Ball)balls.firstElement();
        newBall=new Ball((Ball)originalBall);
        newBall.changeDirection(new Direction(1,1));
        newBall.setPosition(new Point(originalBall.getPosition()));
        balls.addElement(newBall);
        elements.addElement(newBall);
    }
    public void handleAction(Ball parB){
        ownerGame.getActionManager().ballMotion(this,parB);
    }
    public void handleAction(Fire parF){
        if(!fManager.isAlive())
            fManager.start();
        ownerGame.getActionManager().fireMotion(this,parF);
    }    
}