/*
The template class that is used as a base for all the objects that interact with
each other in a game.

Copyright(c) Group Seven.
Added to the OO-Ball in the 1.0 version.
Additional information is in API Reference in our web page.
*/

import java.awt.*;
import java.awt.image.*;

public abstract class ObjectTemplate
{
    protected Point position=new Point(10,10);
    protected Image imageOfObject;
    protected Component owner;
    protected Dimension dim=new Dimension(10,10);
    protected LevelTemplate level;    
    //constructor
    //gets the x and y coordinates for the position
    //and the owner of the object on which it will be drawn
    ObjectTemplate(int parX, int parY, Component parOwner, LevelTemplate parLevel){
        position=new Point(parX, parY);
        owner=parOwner;
        level=parLevel;
    }
    ObjectTemplate(){
    }    
    //sets the x coordinate of the position of object
    public void setX(int parX){
        position.x=parX;
    }
    
    //sets the y coordinate of the position of object
    public void setY(int parY){
        position.y=parY;
    }
    
    //returns the x coordinate of the current position
    public int getX(){
        return position.x;
    }
    public int getActualX(){
        return getX()-level.position.x-Wall.SIZE;
    }
    public int getActualY(){
        return getY()-level.position.y-Wall.SIZE;
    }
    //returns the y coordinate of the current position
    public int getY(){
        return position.y;    
    }
    
    //sets the position of object on the owner
    public void setPosition(Point parPosition){
        position=parPosition;
    }
    
    //sets the position of the object on the owner
    public void setPosition(int parX, int parY){
        position.x=parX;
        position.y=parY;
    }
    
    //gets the current position of the object
    public Point getPosition(){
        return position;
    }
    
    //sets the height of the object
    public void setHeight(int parHeight){
        dim.height=parHeight;
    }
    
    //gets the current height of the object
    public int getHeight(){
        return dim.height;            
    }
    
    //sets the width of the object
    public void setWidth(int parWidth){
        dim.width=parWidth;
    }
    
    //gets the current width
    public int getWidth(){
        return dim.width;
    }
    
    //sets the dimensions of the object
    public void setDimension(Dimension parDim){
        dim=parDim;
    }
    
    //sets the dimensions of the object
    public void setDimension(int parHeight, int parWidth){
        dim.height=parHeight;
        dim.width=parWidth;
    }
    
    //gets the current dimension
    public Dimension getDimension(){
        return dim;
    }
    
    //draws the object onto the given graphics
    public void draw(Graphics parG){
        try{
            parG.drawImage(imageOfObject, position.x, position.y, dim.width, dim.height, owner);
        }
        catch(Exception e){
            System.out.println("Error in drawing object.");
        }
    }
    
    //change the current image of object
    public void changeImage(Image parImageOfObject){
        imageOfObject=parImageOfObject;
    }
    
    public boolean intersects(ObjectTemplate parO){
        Rectangle locRect1,locRect2;
        locRect1=new Rectangle(position, dim);
        locRect2=new Rectangle(parO.position, parO.dim);
        return locRect1.intersects(locRect2);
    }
    
    public void finalize(){
        try{
            super.finalize();
        }catch(java.lang.Throwable e){
            System.out.println("ERRO1: Error in finalizing object!");
        }
    }
    
//The following segment is removed from the code because it could not be completely implemented
//The part checks for the transparent pixels of the objects and if the intersecting points
//only consist of them, it returns false for the intersects value. The code is nearly complete,
//the only problem is finding the int value of transparency in RGB color.
  /*  public boolean intersects(ObjectTemplate parO){
        Rectangle locRect1,locRect2;
        locRect1=new Rectangle(position, dim);
        locRect2=new Rectangle(parO.position, parO.dim);
        if(locRect1.intersects(locRect2)){
		return compare(parO);
		
	  }
		return false;
		
    }

	private boolean handlesinglepixel(int x, int y, int pixel, ObjectTemplate parO) {
		int alpha = (pixel >> 24) & 0xff;	
		int red   = (pixel >> 16) & 0xff;
		int green = (pixel >>  8) & 0xff;	
		int blue  = (pixel      ) & 0xff;
		if(alpha!=0){
			int[] pixels = new int[1];
			PixelGrabber pg = new PixelGrabber(parO.imageOfObject, x+getX()-parO.getX(), y+getY()-parO.getY(), 1, 1, pixels, 0, 1);
			if(pixels[0]!=alpha)
					return true;

		}
		return false;
	// Deal with the pixel as necessary... 
	}
 	
	private boolean handlepixels(Image img, int x, int y, int w, int h, ObjectTemplate parO) {
		int[] pixels = new int[w * h];
		PixelGrabber pg = new PixelGrabber(img, x, y, w, h, pixels, 0, w);
//		int trans=pg.getColorModel().getTransparency();
		try {
		    pg.grabPixels();	} catch (InterruptedException e) {
		    System.err.println("interrupted waiting for pixels!");	    return false;	}
		if ((pg.getStatus() & ImageObserver.ABORT) != 0) {
		    System.err.println("image fetch aborted or errored");	    return false;	}
		for (int j = 0; j < h; j++) {	    for (int i = 0; i < w; i++) {
			if(handlesinglepixel(x+i, y+j, pixels[j * w + i],parO))
			return true;	    }	} 
		return false;
	}
	public boolean compare(ObjectTemplate parO){
		return handlepixels(imageOfObject,0,0,dim.width,dim.height,parO);
	}
*/
}