/*
Paddle used to avoid the ball from falling down in a level.

Copyright(c) Group Seven.
Added to the OO-Ball in the 1.0 version.
Additional information is in API Reference in our web page.
*/

import java.awt.*;
import java.awt.image.*;

public class Paddle extends TypedObject
{
    private Image wholeImage;
    private static final String imageFile="Images/Paddle.gif";
    static final int NORMAL_SIZE=70, SMALL=50, LARGE=90, V_LARGE=110;
    static final int HEIGHT=20, EDGE_DISTANCE=10;
    static final int FREEZE_TIME=200;
    static final int HOLD=1, LASER=2, SPLIT=3, NORMAL=0;
    private boolean verticalEnabled=false, frozen=false;
    private int size=NORMAL_SIZE;
    private ImageProducer producer;
    
    
    Paddle(int parX,int parY, Component parOwner, LevelTemplate parLevel){
        super(parX,parY,parOwner, parLevel);
        try{
            wholeImage=parOwner.getToolkit().getImage(imageFile);
        try {
            MediaTracker tracker = new MediaTracker(owner);
            tracker.addImage(wholeImage, 0);
            tracker.waitForID(0);
        }
        catch ( Exception e ) {}

        }
        catch(Exception e){
            System.out.println("Image file for paddle object could not be found! [image/Paddle.gif]");
        }
        wholeImage.flush();
        setType(HOLD);
        setHeight(HEIGHT);
        setWidth(NORMAL_SIZE);
    }
    public void freeze(){
        frozen=true;
        try{
            new Thread().sleep(FREEZE_TIME);
        }catch(Exception e){}
        frozen=false;
    }
    public boolean isFrozen(){
	return frozen;
    }
    public boolean isVerticalEnabled(){	
	return verticalEnabled;
    }
    public void verticalMotionEnable(boolean parEnable){
        verticalEnabled=parEnable; 
    }
    public void setType(int parType){
        if(type==parType)
            return;
        CropImageFilter locCrop;
        type=parType;
        switch(parType){
            case HOLD:
                locCrop=new CropImageFilter(79,0,79,37);
                break;
            case LASER:
                locCrop=new CropImageFilter(158,0,79,37);
                break;
            default:
                locCrop=new CropImageFilter(0,0,79,37);
                break;
        }
        producer = new FilteredImageSource(wholeImage.getSource(), locCrop);
        imageOfObject = owner.createImage(producer);
        try {
            MediaTracker tracker = new MediaTracker(owner);
            tracker.addImage(imageOfObject, 0);
            tracker.waitForID(0);
        }
        catch ( Exception e ) {}
    }
    public void shorten(){
        if(type==SPLIT)
            setType(NORMAL);
        if(size==NORMAL_SIZE){
            setWidth(SMALL);
        }else if(size==LARGE){
            setWidth(NORMAL_SIZE);
        }else if(size==V_LARGE){
            setWidth(LARGE);
        }
    }

    public void enlarge(){    
        if(type==SPLIT)
            setType(NORMAL);
        if(size==NORMAL_SIZE){
            setWidth(LARGE);
        }else if(size==LARGE){
            setWidth(V_LARGE);
        }else if(size==SMALL){
            setWidth(NORMAL_SIZE);
        }
    }
    public void draw(Graphics parG){
        try{
            if(type!=Paddle.SPLIT)
			parG.drawImage(imageOfObject, position.x, position.y-5, dim.width, dim.height, owner);
		else{
			parG.drawImage(imageOfObject, position.x-60, position.y-5, dim.width, dim.height, owner);
			parG.drawImage(imageOfObject, position.x+60, position.y-5, dim.width, dim.height, owner);
		}
        }
        catch(Exception e){
            System.out.println("Error in drawing paddle.");
        }
    }   
    public boolean intersects(ObjectTemplate parO){
    	  if(type!=Paddle.SPLIT)
		return super.intersects(parO);
	  Rectangle locRect1,locRect2,locRect3;
        locRect1=new Rectangle(new Point(position.x-60,position.y), dim);
	  locRect2=new Rectangle(new Point(position.x+60,position.y), dim);
        locRect3=new Rectangle(parO.position, parO.dim);
        return (locRect1.intersects(locRect3)||locRect2.intersects(locRect3));
    }
 
}